import math
from math import inf

from torch.utils.data import DataLoader


def exists(x):
    return x is not None


def default(val, d):
    if exists(val):
        return val
    return d() if callable(d) else d


def identity(t, *args, **kwargs):
    return t


def cycle(dl):
    while True:
        for data in dl:
            yield data


def has_int_squareroot(num):
    return (math.sqrt(num) ** 2) == num


def num_to_groups(num, divisor):
    groups = num // divisor
    remainder = num % divisor
    arr = [divisor] * groups
    if remainder > 0:
        arr.append(remainder)
    return arr


def convert_image_to_fn(img_type, image):
    if image.mode != img_type:
        return image.convert(img_type)
    return image


def infinite_dataloader(dl: DataLoader, n_steps: int = inf):
    i = 0
    alive = True
    while alive:
        for batch in dl:
            yield batch
            i += 1
            if i >= n_steps:
                alive = False
                break
